--[[
 Copyright 2008 Geir H. Pettersen
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at
    http://www.apache.org/licenses/LICENSE-2.0
 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
]]

--[[
Name: DemocraticReadyCheck
Revision: $Revision: 53 $
Last update: $Date: 2009-05-02 21:34:30 +0000 (Sat, 02 May 2009) $
Author: Geir H. Pettersen
Email: ghenningp [at] gmail [dot] com
Description: Make your whole party/raid aware of whats happening during and after a readycheck.
]]

----------------------------------
-- Libraries
----------------------------------
DemocraticReadyCheck = LibStub("AceAddon-3.0"):NewAddon("DemocraticReadyCheck", "AceConsole-3.0", "AceEvent-3.0")
----------------------------------
-- Config options
----------------------------------
local options = {
   handler = DemocraticReadyCheck,
   type='group',
   args = {
      win = {
		   type = 'execute',
			name = 'Options Window',
			desc = 'Shows the Options Window',
			func = "OpenWaterfall",
			guiHidden = true,
      },
      enable={
         name="Enable",
         desc="Enables / disables the addon",
         type="toggle",
         get = "GetEnabled",
         set = "SetEnabled",
      },
      messages = {
         type = 'group',
         name = 'Message config',
         args = {
            header = {
               type = "header",
               name = "ReadyCheck messages",
               order = 1,
            },
            initiate = {
               type = "select",
               name = "Initiate message type",
               desc = "Set how ReadyCheck initiate message should be sent",
               get = "GetInitiateType",
               set = "SetInitiateType",
               values = {
                  ["DISABLED"] = "Disabled",
                  ["CHAT"] = "Chat",
                  ["WARN"] = "Warning"
               },
               order = 10,
            },
            confirm = {
               type = "select",
               name = "Confirm message type",
               desc = "Set how ReadyCheck confirm (per character) message should be sent",
               get = "GetConfirmType",
               set = "SetConfirmType",
               values = {
                  ["DISABLED"] = "Disabled",
                  ["CHAT"] = "Chat",
                  ["WARN"] = "Warning"
               },
               order = 20,
            },
            complete = {
               type = "select",
               name = "Complete message type",
               desc = "Set how ReadyCheck complete message should be sent",
               get = "GetCompleteType",
               set = "SetCompleteType",
               values = {
                  ["DISABLED"] = "Disabled",
                  ["CHAT"] = "Chat",
                  ["WARN"] = "Warning"
               },
               order = 30,
            },
            finalstatus = {
               type = "select",
               name = "Final status message type",
               desc = "Set how ReadyCheck final status message should be sent",
               get = "GetFinalStatusType",
               set = "SetFinalStatusType",
               values = {
                  ["DISABLED"] = "Disabled",
                  ["CHAT"] = "Chat",
                  ["WARN"] = "Warning"
               },
               order = 40,
            },
            texts = {
               type = 'group',
               name = 'Message text config',
               desc = 'Configuration of ReadyCheck message texts',
               order = 50,
               args = {
                  header = {
                     type = "description",
                     name = "%s is replaced by character name(s)",
                     order = 1,
                  },
                  initiate = {
                     type = "input",
                     name = "initiate",
                     desc = "Set the readycheck initiate text",
                     usage = "<Your message here>",
                     get = "GetInitiateStr",
                     set = "SetInitiateStr",
                     order = 10,
                  },
                  confirmYes = {
                     type = "input",
                     name = "confirmYes",
                     desc = "Set the readycheck confirmYes text",
                     usage = "<Your message here>",
                     get = "GetConfirmYesStr",
                     set = "SetConfirmYesStr",
                     order = 20,
                  },
                  confirmNo = {
                     type = "input",
                     name = "confirmNo",
                     desc = "Set the readycheck confirmNo text",
                     usage = "<Your message here>",
                     get = "GetConfirmNoStr",
                     set = "SetConfirmNoStr",
                     order = 30,
                  },
                  complete = {
                     type = "input",
                     name = "complete",
                     desc = "Set the readycheck complete text",
                     usage = "<Your message here>",
                     get = "GetCompleteStr",
                     set = "SetCompleteStr",
                     order = 40,
                  },
                  finalStatusNo = {
                     type = "input",
                     name = "finalStatusNo",
                     desc = "Set the readycheck finalStatusNo text",
                     usage = "<Your message here>",
                     get = "GetFinalStatusNoStr",
                     set = "SetFinalStatusNoStr",
                     order = 50,
                  },
                  finalStatusAfk = {
                     type = "input",
                     name = "finalStatusAfk",
                     desc = "Set the readycheck finalStatusAfk text",
                     usage = "<Your message here>",
                     get = "GetFinalStatusAfkStr",
                     set = "SetFinalStatusAfkStr",
                     order = 60,
                  },
                  finalStatusOk = {
                     type = "input",
                     name = "finalStatusOk",
                     desc = "Set the readycheck finalStatusOk text",
                     usage = "<Your message here>",
                     get = "GetFinalStatusOkStr",
                     set = "SetFinalStatusOkStr",
                     order = 70,
                  },
               },
            },
         },
      },
   },
}

local defaults = {
   profile = {
      enabled = true,
      initiateType = "DISABLED",
      confirmType = "DISABLED",
      completeType = "CHAT",
      finalStatusType = "WARN",
      initiateStr = "%s has initiated a ready check.",
      confirmYesStr = "%s is ready.",
      confirmNoStr = "%s is NOT ready.",
      completeStr = "Ready check completed.",
      finalStatusNoStr = "%s is NOT ready.",
      finalStatusAfkStr = "%s is AFK.",
      finalStatusOkStr = "Everyone is ready!",
   },
}

----------------------------------
-- Initialization
----------------------------------

----------------------------------
-- DemocraticReadyCheck object
----------------------------------

----------------------
-- Addon maintainance
----------------------
function DemocraticReadyCheck:OnInitialize()
    -- Called when the addon is loaded
   self.db = LibStub("AceDB-3.0"):New("DemocraticReadyCheckDB", defaults, "Default")
   options.args.profile = LibStub("AceDBOptions-3.0"):GetOptionsTable(self.db)
	LibStub("AceConfig-3.0"):RegisterOptionsTable("DemocraticReadyCheck", options)
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DemocraticReadyCheck", "DemocraticReadyCheck")
   self:RegisterChatCommand("democraticreadycheck", "ChatCommand")
   self:RegisterChatCommand("drc", "ChatCommand")
end

function DemocraticReadyCheck:OnEnable()
   -- Called when the addon is enabled
   if (self:GetEnabled()) then
      self:RegisterEvents()
   end
end

function DemocraticReadyCheck:OnDisable()
   -- Called when the addon is disabled
end

function DemocraticReadyCheck:OpenWaterfall()
   LibStub("AceConfigDialog-3.0"):Open("DemocraticReadyCheck")
end

function DemocraticReadyCheck:ChatCommand(input)
   LibStub("AceConfigCmd-3.0").HandleCommand(DemocraticReadyCheck, "drc", "DemocraticReadyCheck", input)
end

----------------------
-- Value get/setters
----------------------
function DemocraticReadyCheck:GetEnabled(info)
   return self.db.profile.enabled
end

function DemocraticReadyCheck:SetEnabled(info, newValue)
   self.db.profile.enabled = newValue
   if (newValue) then
      self:RegisterEvents()
   else
      self:UnregisterEvents()
   end
end

function DemocraticReadyCheck:GetInitiateType(info)
   return self.db.profile.initiateType
end

function DemocraticReadyCheck:SetInitiateType(info, newValue)
   self.db.profile.initiateType = newValue
end

function DemocraticReadyCheck:GetConfirmType(info)
   return self.db.profile.confirmType
end

function DemocraticReadyCheck:SetConfirmType(info, newValue)
   self.db.profile.confirmType = newValue
end

function DemocraticReadyCheck:GetCompleteType(info)
   return self.db.profile.completeType
end

function DemocraticReadyCheck:SetCompleteType(info, newValue)
   self.db.profile.completeType = newValue
end

function DemocraticReadyCheck:GetFinalStatusType(info)
   return self.db.profile.finalStatusType
end

function DemocraticReadyCheck:SetFinalStatusType(info, newValue)
   self.db.profile.finalStatusType = newValue
end

function DemocraticReadyCheck:GetInitiateStr(info)
   return self.db.profile.initiateStr
end

function DemocraticReadyCheck:SetInitiateStr(info, newValue)
   self.db.profile.initiateStr = newValue
end

function DemocraticReadyCheck:GetConfirmYesStr(info)
   return self.db.profile.confirmYesStr
end

function DemocraticReadyCheck:SetConfirmYesStr(info, newValue)
   self.db.profile.confirmYesStr = newValue
end

function DemocraticReadyCheck:GetConfirmNoStr(info)
   return self.db.profile.confirmNoStr
end

function DemocraticReadyCheck:SetConfirmNoStr(info, newValue)
   self.db.profile.confirmNoStr = newValue
end

function DemocraticReadyCheck:GetCompleteStr(info)
   return self.db.profile.completeStr
end

function DemocraticReadyCheck:SetCompleteStr(info, newValue)
   self.db.profile.completeStr = newValue
end

function DemocraticReadyCheck:GetFinalStatusNoStr(info)
   return self.db.profile.finalStatusNoStr
end

function DemocraticReadyCheck:SetFinalStatusNoStr(info, newValue)
   self.db.profile.finalStatusNoStr = newValue
end

function DemocraticReadyCheck:GetFinalStatusAfkStr(info)
   return self.db.profile.finalStatusAfkStr
end

function DemocraticReadyCheck:SetFinalStatusAfkStr(info, newValue)
   self.db.profile.finalStatusAfkStr = newValue
end

function DemocraticReadyCheck:GetFinalStatusOkStr(info)
   return self.db.profile.finalStatusOkStr
end

function DemocraticReadyCheck:SetFinalStatusOkStr(info, newValue)
   self.db.profile.finalStatusOkStr = newValue
end
----------------------
-- Business
----------------------
function DemocraticReadyCheck:RegisterEvents()
   self:RegisterEvent("READY_CHECK")
   self:RegisterEvent("READY_CHECK_CONFIRM")
   self:RegisterEvent("READY_CHECK_FINISHED")
end

function DemocraticReadyCheck:UnregisterEvents()
   self:UnregisterEvent("READY_CHECK")
   self:UnregisterEvent("READY_CHECK_CONFIRM")
   self:UnregisterEvent("READY_CHECK_FINISHED")
end

function DemocraticReadyCheck:READY_CHECK(eventName, characterName, arg2)
   -- Called when a ready check is performed
   if self:isApplicable() then
      self:sendMessage(string.format(self.db.profile.initiateStr, characterName), self.db.profile.initiateType)
      self.notReady = {}
      self.ready = {}
      self.afk = self:getFullGroup()
      self:afkRemove(characterName)
   end
end

function DemocraticReadyCheck:READY_CHECK_CONFIRM(eventName, prtyMN, status)
   -- Called when a player confirms ready status
   if self:isApplicable() then
      local unitName = UnitName(self:getUnitPrefix() .. prtyMN)
      if (not unitName) then
         unitName = UnitName("player")
      end
      self:afkRemove(unitName)
      local stStr
      if status == 1 then
         stStr = self.db.profile.confirmYesStr
         table.insert(self.ready, unitName)
      else
         stStr = self.db.profile.confirmNoStr
         table.insert(self.notReady, unitName)
      end
      self:sendMessage(string.format(stStr, unitName), self.db.profile.confirmType)
   end
end

function DemocraticReadyCheck:READY_CHECK_FINISHED(eventName)
   -- Called when ready check completes
   if self:isApplicable() then
      local stStr = nil
      local charListStr = nil
      if (table.getn(self.notReady) > 0) then
         stStr = self.db.profile.finalStatusNoStr
         charListStr = strjoin(", ", unpack(self.notReady))
      elseif (table.getn(self.afk) > 0) then
         stStr = self.db.profile.finalStatusAfkStr
         charListStr = strjoin(", ", unpack(self.afk))
      else
         stStr = self.db.profile.finalStatusOkStr
      end
      -- Notify final status
      self:sendMessage(string.format(stStr, charListStr), self.db.profile.finalStatusType)
      -- Notify complete
      self:sendMessage(self.db.profile.completeStr, self.db.profile.completeType)

      self.notReady = nil
      self.ready = nil
      self.afk = nil
   end
end

function DemocraticReadyCheck:isApplicable()
   if UnitInRaid("player") then
      return IsRaidOfficer()
   else      
      return IsPartyLeader()
   end
end

function DemocraticReadyCheck:sendMessage(message, sendType)
   -- Called when something has happened that may result in a message
   if (sendType == "CHAT") then
      SendChatMessage(message, self:getUnitPrefix())
   elseif (sendType == "WARN") then
      SendChatMessage(message, "RAID_WARNING")
   end
end

function DemocraticReadyCheck:getUnitPrefix()
   if UnitInRaid("player") then
      return "RAID"
   else
      return "PARTY"
   end
end

function DemocraticReadyCheck:afkRemove(name)
   for i=1, table.getn(self.afk) do
      if (name == self.afk[i]) then
         table.remove(self.afk, i)
         break
      end
   end
end

function DemocraticReadyCheck:getFullGroup()
   local groupCount
   local fullGroup = {}
   if UnitInRaid("player") then
      groupCount = GetNumRaidMembers()
   else
      groupCount = GetNumPartyMembers()
   end
   for i=1, groupCount do
      local unitName = UnitName(self:getUnitPrefix() .. i)
      table.insert(fullGroup, unitName)
   end
   return fullGroup
end